<?php

namespace Drupal\simple_global_filter\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class GlobalFilterForm.
 */
class GlobalFilterForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $global_filter = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $global_filter->label(),
      '#description' => $this->t("Label for the Global filter."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $global_filter->id(),
      '#machine_name' => [
        'exists' => '\Drupal\simple_global_filter\Entity\GlobalFilter::load',
      ],
      '#disabled' => !$global_filter->isNew(),
    ];

    $vocabularies = \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->loadMultiple();
    $options = [];
    foreach ($vocabularies as $id => $vocabulary) {
      $options[$id] = $vocabulary->label();
    }
    $form['vocabulary_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Vocabulary'),
      '#description' => $this->t('Select the taxonomy vocabulary related to this global filter.'),
      '#required' => TRUE,
      '#options' => $options,
      '#default_value' => $global_filter->getVocabulary(),
      '#ajax' => [
        'callback' => '::termsVocabularyCallback',
        'wrapper' => 'default-value-container',
      ],
    ];

    $form['wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'default-value-container'],
    ];

    // Default value: first check if
    $default_value = $global_filter->getDefaultValue()?
          $global_filter->getDefaultValue() : ($form_state->getValue('default_value')?
              $form_state->getValue('default_value') : '' 
              );
    
    if (($vocabulary_name = $form_state->getValue('vocabulary_name')) ||
        ($vocabulary_name = $global_filter->getVocabulary())) {
      
      $options = static::getTermsFromVocabulary($vocabulary_name);
      $disabled = FALSE;
      $title = $this->t('Default value for vocabulary @vocabulary', 
        ['@vocabulary' => \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->load($vocabulary_name)->label()]);
    }
    else {
      $options = [];
      $disabled = TRUE;
      $title = $this->t('First choose a vocabulary.');
    }

    $form['wrapper']['default_value'] = [
      '#type' => 'select',
      '#title' => $title,
      '#description' => $this->t('If there has not been selected any value yet, this value will be returned.'),
      '#options' => $options,
      '#default_value' => $default_value,
      '#disabled' => $disabled,
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $global_filter = $this->entity;
    $status = $global_filter->save();

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label Global filter.', [
          '%label' => $global_filter->label(),
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label Global filter.', [
          '%label' => $global_filter->label(),
        ]));
    }
    $form_state->setRedirectUrl($global_filter->toUrl('collection'));
  }

  public static function getTermsFromVocabulary($vocabulary_name) {
    $options = [];
    foreach (\Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($vocabulary_name, 0, NULL, TRUE) as $term) {
      $options[$term->id()] = $term->label();
    }
    return $options;
  }

  public function termsVocabularyCallback(array $form, FormStateInterface $form_state) {
    return $form['wrapper'];
  }
}
